#!/usr/bin/perl -w

use Tk;
use AdminAgent;
use Tk::Dialog; 

################################################################################
# Lists to hold Scrolling Lists Information
local @SubjectList;
local @ParamList;
local @CommandList;
local @ScriptIdList;
local @UserList;

################################################################################
# Localize Widgets
local $main;                           # Main Window 
local $menubar;                        # Menu Bar for Main Window & his members
local $file_menu;                      #
local $edit_menu;                      #
local $ServiceListbox;                 # Listbox for Services

&InitDB();                             # Open DataBases

$main = MainWindow->new();
$main->wm("title","Admin Agent Administration");
$main->wm("geometry","600x400");
$main->wm("iconname","Agent");


# Create a horizontal space at the top of the window
# for the menu to live in
$menubar = $main->Frame(-relief        => "raised",
						-borderwidth   => 2)
                ->pack (-anchor        => "nw",
						-fill          => "x");

$file_menu = $menubar->Menubutton(-text     =>"File",
								  -underline   => 1 ,
								  -menuitems   => [
								  [ Button => "Load Configuration", -command => \&LoadConfig ],
								  [ Button => "Clear List",-command => \&ClearList ],
								  [ Button => "Quit", -command => \&Quit ] ] )
                     ->pack(-side              => "left");
$edit_menu = $menubar->Menubutton(-text     =>"Edit",
								  -underline   => 1 ,
								  -menuitems   => [
								  [ Button => "Delete Selected Service", -command => \&DeleteService ],
								  [ Button => "Add Service",-command => \&AddService],
								  [ Button => "Users of Selected Service",-command => \&UserAdmin],
								  ])
                     ->pack(-side              => "left");


$ListBoxLabel = "Subject        Command       Parameter" ;
$ServiceListbox = $main->ScrlListbox(-label => $ListBoxLabel)
				->pack(-fill         => "both",
					   -expand       => "yes");
MainLoop;



sub Quit {
    exit(0);
}

sub LoadConfig {
	my $Count = 0 ;

	&ClearList();
	foreach $Subject ( &list_Actions() ) {
			($Action,$Param,$ScriptId) = get_Actions($Subject,"Reset") ;
			while (  defined($Param) & defined($ScriptId) ) {
                $SubjectList[$Count]  = $Subject;
				$ParamList[$Count]    = $Param ;
				$CommandList[$Count]  = $Action ;
				$ScriptIdList[$Count] = $ScriptId;
				$Count++ ;
				$Line = sprintf("%s   %s    %s    %s",$Subject,$Action,$Param,get_Script($ScriptId)) ;
                $ServiceListbox->insert("end",$Line) ;
			    ($Action,$Param,$ScriptId) = get_Actions($Subject,"Next") ;
			}
	}
}

sub InitDB {
	$Access = "Write" ;
    &open_database($Access) ;
}

sub ClearList {
    my $ListLenght = $ServiceListbox->size ;
    $ServiceListbox->delete(0,$ListLenght) ;
    splice(@SubjectList,1,$#SubjectList);
    splice(@ParamList,1,$#ParamList);
    splice(@CommandList,1,$#CommandList);
    splice(@ScriptIdList,1,$#ScriptIdList);
}

sub DeleteService {

    my $SelectedElement = ($ServiceListbox->curselection())[0] ;
	if ( defined($SelectedElement) ) {
	   if ( Warning("Will You really delete this Service ? ") ) {
	        &del_Service($SubjectList[$SelectedElement],
						 $CommandList[$SelectedElement],
						 $ParamList[$SelectedElement]);
	        &LoadConfig();
	        return(0);
	   }
	} else {
	   Error("You havn't anything selected");
	   return(-1);
	}
}

sub Error {
    my ( $ErrorText ) = @_ ;
    $error_dialog = $main->Dialog( -title   => "Error",
							       -text    => $ErrorText,
								   -bitmap  => "error",
							       -buttons => ["OK"] );
    $error_dialog->Show();
}

sub Warning {
    my ( $WarningText ) = @_ ;
	my $okButton = "Delete the Service" ;
	my $cancelButton = "Cancel" ;
	my $warning_dialog ;

    $warning_dialog = $main->Dialog( -title   => "Warning",
							       -text    => $WarningText,
								   -bitmap  => "questhead",
							       -buttons => [$okButton,$cancelButton] );

	if ( $warning_dialog->Show() eq $okButton ) {
	   return 1 ;
	} else {
	   return 0 ;
	}
}

sub AddService {
	my $SubjectTxt = "Subject:        ";
	my $CommandTxt = "Command:        ";
	my $ParamText  = "Parameter:      ";
	my $PrgrmText  = "Programm:       ";

	my $AddServiceWindow ;
	my $CloseButton ;
	my $ResetButton ;
	my $InsertButton ;

	my $ProgramLabel ;
	my $SubjectLabel ;
	my $CommandLabel ;
	my $ParamLabel ;

	my $ProgramEntry ;
	my $SubjectEntry ;
	my $CommandEntry ;
	my $ParamEntry ;

    $AddServiceWindow = $main->Toplevel();
    $AddServiceWindow->wm("title","Add New Service");
    $AddServiceWindow->wm("iconname","Agent");

	$Frame1 = $AddServiceWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-anchor        => "nw",
									  -side          => "top",
						              -fill          => "x");
    $SubjectLabel = $Frame1->Label(-text         =>$SubjectTxt)
						   ->pack(-side => "left");
	$SubjectEntry = $Frame1->Entry(-textvariable => \$Subject )
						   ->pack(-side => "left");

	$Frame2 = $AddServiceWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");
    $CommandLabel = $Frame2->Label(-text         =>$CommandTxt)
						   ->pack(-side => "left");
	$CommandEntry = $Frame2->Entry(-textvariable => \$Command )
						   ->pack(-side => "left");

	$Frame3 = $AddServiceWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");
    $ParamLabel = $Frame3->Label(-text         =>$ParamText)
				   ->pack(-side => "left");
	$ParamEntry = $Frame3->Entry(-textvariable => \$Param )
				   ->pack(-side => "left");
	$Frame4 = $AddServiceWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");
    $ProgramLabel = $Frame4->Label(-text         =>$PrgrmText)
						   ->pack(-side => "left");
	$ProgramEntry = $Frame4->Entry(-textvariable => \$Program ,
								   -width        => 40 )
						   ->pack(-side => "left");
	$Frame5 = $AddServiceWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");
	$CloseButton = $Frame5->Button(-text=>"Close",
								   -command=>sub {
												  $AddServiceWindow->destroy();
								                  })
                          ->pack(-side        => "left");
    $ResetButton = $Frame5->Button(-text=>"Reset",
								   -command=>sub {
												   $Subject = "";
												   $Command = "";
												   $Param = "" ;
												   $Program = "" ;
								                  })
                          ->pack(-side        => "left");
	$InsertButton = $Frame5->Button(-text=>"Insert",
								   -command=>sub {
												  print "$Subject \n";
												  print "$Command \n";
												  print "$Param \n";
												  print "$Program \n";
												  &add_Service($Subject,$Command,$Param,$Program) ;
												  &LoadConfig();
								                  })
                          ->pack(-side        => "left");

}

sub UserAdmin {
	my $Frame1 ;
	my $Frame2 ;
	my $ScriptID ;

	my $CloseButton;
	my $InsertButton;
	my $DeleteButton;

    my $SelectedElement = ($ServiceListbox->curselection())[0] ;

	if ( ! defined($SelectedElement) ) {
	   Error("You havn't anything selected");
	   return(-1);
	}
	$ScriptID = $ScriptIdList[$SelectedElement] ;
    
    $UserAdminWindow = $main->Toplevel();
    $UserAdminWindow->wm("title","UserAdministration");
    $UserAdminWindow->wm("iconname","Agent");

    $Frame1 = $UserAdminWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");
    $Frame2 = $UserAdminWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-side          => "top",
						              -fill          => "x");

    $ListBoxLabel = "UserList" ;
    $UserListbox = $Frame1->ScrlListbox(-label => $ListBoxLabel)
				    ->pack(-fill         => "both",
					       -expand       => "yes");
	$CloseButton = $Frame2->Button(-text=>"Close",
								   -command=>sub {
												  # b4 we close we have to clean up
												  $ServiceListbox->selectionSet($SelectedElement);
												  $UserAdminWindow->destroy();
								                  })
                          ->pack(-side        => "left");
	$InsertButton = $Frame2->Button(-text=>"Insert User",
								   -command=>sub {
												  &InsertUser($UserListbox,$ScriptID);
								                  })
                          ->pack(-side        => "left");
	$DeleteButton = $Frame2->Button(-text=>"Delete User",
								   -command=>sub {
												  &DeleteSelectedUser($UserListbox,$ScriptID);
								                  })
                          ->pack(-side        => "left");

    &UpdateUserList($UserListbox,$ScriptID) ;
}

sub UpdateUserList {
    my ($UserListbox,$ScriptID) = @_ ;
    my $ListLenght = $UserListbox->size ;

    $UserListbox->delete(0,$ListLenght) ;
    splice(@UserList,1,$#UserList);
    for $UserId ( list_Users($ScriptID) ) {
		$UserListbox->insert("end",$UserId) ;
		push(@UserList,$UserId);
	}
}

sub InsertUser {
   my ($UserListbox,$ScriptId) = @_ ;
   my $Frame1 ;
   my $Frame2 ;
   my $UserId ;
   my $CloseButton ;
   my $InsertButton ;
   my $UserIdLabel ;
   my $UserIdEntry ;

   $UserAddWindow = $main->Toplevel();
   $UserAddWindow->wm("title","Add a new user");
   $UserAddWindow->wm("iconname","Agent");

   $Frame1 = $UserAddWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-anchor        => "nw",
									  -side          => "top",
						              -fill          => "x");
   $Frame2 = $UserAddWindow->Frame(-relief => "flat",
									   -borderwidth => 3 )
                               ->pack(-anchor        => "nw",
									  -side          => "top",
						              -fill          => "x");
   $UserIdLabel = $Frame1->Label(-text         =>"UserId")
                         ->pack(-side => "left");
   $UserIdEntry = $Frame1->Entry(-textvariable => \$UserId )
                         ->pack(-side => "left");

   $CloseButton = $Frame2->Button(-text=>"Close",
                                  -command=>sub {
												  $UserAddWindow->destroy();
								                  })
                          ->pack(-side        => "left");
   $InsertButton = $Frame2->Button(-text=>"Insert User",
								   -command=>sub {
												  if ( defined ($UserId) ) {
													  print "Adding $ScriptId $UserId \n" ;
												      &add_Auth($UserId,$ScriptId);
												      &UpdateUserList($UserListbox,$ScriptId) ;
												      $UserAddWindow->destroy();
												  }
								                  })
                          ->pack(-side        => "left");
}

sub DeleteSelectedUser {
    my ($UserListbox,$ScriptID) = @_ ;
	my $UserId = $UserList[($UserListbox->curselection())[0]] ; 
	
	if ( defined($UserId) ) { 
	   if ( Warning("Will You really delete this User ? ") ) { 
          &del_Auth($UserId,$ScriptID);
	      &UpdateUserList($UserListbox,$ScriptID) ;
	      return(0);
	   }
	} else {
	   Error("You havn't anything selected");
	   return(-1);
	}
}
